SystemOrganization addCategory: #'DAO-Satellite'!

ProtoObject subclass: #MD5MacGenerator
	instanceVariableNames: 'sequenceNumber serializedCommandCall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MD5MacGenerator class methodsFor: 'key' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
key

	"
	'Generate a random binary key' asByteArray adding 16r00 up to 32 len
	"
	
	^ #[71 101 110 101 114 97 116 101 32 97 32 114 97 110 100 111 109 32 98 105 110 97 114 121 32 107 101 121 00 00 00 00]! !

!MD5MacGenerator methodsFor: 'initialization' stamp: 'HernanWilkinson 3/25/2013 14:54'!
initializeFor: aSerializedCommandCall with: aSequenceNumber

	serializedCommandCall := aSerializedCommandCall.
	sequenceNumber := aSequenceNumber.
! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'HernanWilkinson 3/25/2013 15:36'!
macFrom: md5

	| mac |
	
	mac := md5 first: 3.
	mac swap: 1 with: 3.
	^ mac! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'NahuelGarbezza 5/18/2013 09:40'!
md5For: aStream

	aStream reset.
	^ MD5 hashStream: aStream! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
streamToCalculateMD5

	| stream |
	stream := ReadWriteStream on: ByteArray new.
	stream
		nextPutAll: self key;
		nextPut: (sequenceNumber bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 2) bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 3) bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 4) bitAnd: 16rFF);
		nextPutAll: serializedCommandCall.
	^ stream! !

!MD5MacGenerator methodsFor: 'evaluating' stamp: 'HernanWilkinson 3/25/2013 15:21'!
value

	| stream md5 mac |
	
	stream := self streamToCalculateMD5.
	md5 := self md5For: stream.
	mac := self macFrom: md5.
	
	^ mac
	! !

TestCase subclass: #CommandEncoderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandEncoderTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
testEncodingNameCommandRequestShouldReturnCorrectString

	|encoder nameCommand resultingByteArray |
	
	encoder := CommandEncoder new.
	nameCommand :=  GetNameCommand newWithReceiver: (SatelliteTest memorySubsystem) andId: 0. 
	
	resultingByteArray := encoder encodeWith: 'FFF' andSequenceNumber: SequenceNumber new andCommand: nameCommand.
	
	self assert: resultingByteArray equals:  #[70 70 70 0 0 1 1 0]! !

!CommandEncoderTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testEncodingSetMatrixKeyRequestShouldReturnCorrectString

	|encoder  command resultingByteArray |
	
	encoder := CommandEncoder new.
	command :=  ThermalSetMatrixKeyCommand newWithReceiver: SatelliteTest thermalSubsystem andId: 32.
	command addParameterValue: 1.
	resultingByteArray := encoder encodeWith: 'FFF' andSequenceNumber: SequenceNumber new andCommand: command.
	
	
	self assert: resultingByteArray equals:  #[70 70 70 0 0 1 3 32 0 0 0 1]! !

!CommandEncoderTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
testEncodingTelemetryRequestShouldReturnCorrectString

	| encoder  nameCommand resultingByteArray |
	
	encoder := CommandEncoder new.
	nameCommand :=  ThermalTelemetryCommand newWithReceiver: (SatelliteTest memorySubsystem) andId: 1. 
	resultingByteArray := encoder encodeWith: 'FFF' andSequenceNumber: SequenceNumber new andCommand: nameCommand.
	
	
	self assert: resultingByteArray equals:  #[70 70 70 0 0 1 1 1]! !

TestCase subclass: #CommandParameterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandParameterTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:32'!
testDeserializeNegativeSigned16BitInteger
	| serializedValue parameter streamDeLectura |
	
	serializedValue := #[1 1].
	streamDeLectura := ReadStream on: serializedValue.
	parameter := SignedIntegerCommandParameter new: 16.
	
	self assert: (parameter deserialize: streamDeLectura) equals: -1.! !

!CommandParameterTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:35'!
testDeserializePositiveSigned16BitInteger
| serializedValue parameter streamDeLectura |
	
	serializedValue := #[0 1].
	streamDeLectura := ReadStream on: serializedValue.
	parameter := SignedIntegerCommandParameter new: 16.
	
	self assert: (parameter deserialize: streamDeLectura) equals: 1.! !

!CommandParameterTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:31'!
testSerializeString8
	| parameter |
	
	parameter := StringCommandParameter new: 8.
	parameter value: 'Hello'.
	
	self assert: (parameter serialize) equals: #[5 72 101 108 108 111].! !

!CommandParameterTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:27'!
testSerializeUnsigned32BitInteger
	| parameter |
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameter value: 1.
	
	self assert: (parameter serialize) equals: #[0 0 0 1].! !

TestCase subclass: #CommandTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandTest class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:54'!
thermalSubsystemNameCommand

	^  GetNameCommand newWithReceiver: SatelliteTest thermalSubsystem andId: 0.
	
! !

!CommandTest class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:57'!
thermalSubsystemTelemetryCommand

	^  ThermalTelemetryCommand newWithReceiver: SatelliteTest thermalSubsystem andId: 1.
	
! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:09'!
testDeserializingArrayOfUnsigned8BitIntegerShouldReturnTheDecodedArray

	| encodedResponse decodedResponse readAddressCommand array |
	
	"setup"
	"001 sequence number, 1 first byte, 2 second"
	encodedResponse := #[0 0 1 1 2].

	readAddressCommand := MemoryReadAddressCommand new.
	
	"exercise"	
	decodedResponse := readAddressCommand deserialize: encodedResponse.
	
	"verify"
	self assert: (decodedResponse data size) equals: 1.
	
	array := (decodedResponse data at: 1).
	
	self assert: array size equals: 2.
	self assert: (array at: 1) equals: 1.
	self assert: (array at: 2) equals: 2.

! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 19:46'!
testDeserializingThermalNameResponseShouldReturnThermal

	| encodedResponse decodedResponse getNameCommand |

	"001 sequence number, lo que le sigue THERMAL codificado"
	encodedResponse := #[0 0 1 84 72 69 82 77 65 76].
	
	getNameCommand := GetNameCommand new.
	
	decodedResponse := getNameCommand deserialize: encodedResponse.
	
	self assert: (decodedResponse data size) equals: 1.
	self assert: (decodedResponse data at: 1) equals: 'THERMAL'.! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 19:53'!
testDeserializingThermalSetMatrixKeyResponseShouldReturnVoid

	| encodedResponse thermalSetMatrixKeyCommand decodedResponse |
	
	"001 sequence number, then empty response"
	encodedResponse := #[0 0 1].
	
	thermalSetMatrixKeyCommand :=  ThermalSetMatrixKeyCommand new.
	
	decodedResponse :=  thermalSetMatrixKeyCommand deserialize: encodedResponse.
	
	self assert: (decodedResponse class) equals: VoidCommandResponse.
	
! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:37'!
testDeserializingThermalShortTelemetryResponseShouldReturnThermalShortTelemetry

	| encodedResponse thermalShortTelemetryCommand decodedResponse |
	
	"001 sequence number, 1 battery temp"
	encodedResponse := #[0 0 1 1].
	thermalShortTelemetryCommand :=  ThermalShortTelemetryCommand new.
	
	decodedResponse :=  thermalShortTelemetryCommand deserialize: encodedResponse.
	
	self assert: (decodedResponse data at: 1) equals: 1.
	
! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 15:06'!
testDeserializingThermalTelemetryResponseContainingNegativeNumbersShouldDeserializeOK

	| encodedResponse decodedResponse thermalTelemetryCommand |
	
	"001 sequence number, 1 1 battery temp, 1 2 radio temp"
	encodedResponse := #[0 0 1 1 1 1 2].

	thermalTelemetryCommand := ThermalTelemetryCommand new.
		
	decodedResponse := thermalTelemetryCommand deserialize: encodedResponse.
	
	self assert: (decodedResponse data at: 1) equals: -1.
	self assert: (decodedResponse data at: 2) equals: -2.
! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 15:03'!
testDeserializingThermalTelemetryResponseShouldReturnThermalTelemetry

	| encodedResponse decodedResponse thermalTelemetryCommand |
	
	"001 sequence number, 0 1 battery temp, 0 2 radio temp"
	encodedResponse := #[0 0 1 0 1 0 2].

	thermalTelemetryCommand := ThermalTelemetryCommand new.
		
	decodedResponse := thermalTelemetryCommand deserialize: encodedResponse.
	
	self assert: (decodedResponse data at: 1) equals: 1.
	self assert: (decodedResponse data at: 2) equals: 2.
! !

!CommandTest methodsFor: 'deserialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:19'!
testDeserializingUsigned32BitIntegerAndArrayOfUnsigned8BitIntegerShouldDeserializeOK

	| encodedResponse decodedResponse readLongAddressCommand array firstElement |
	
	"setup"
	"001 sequence number, 0 0 0 1 first element of the response,  then 1 and 2 are the elements of the array"
	encodedResponse := #[0 0 1 0 0 0 1 1 2].

	readLongAddressCommand := MemoryReadLongAddressCommand new.
	
	"exercise"	
	decodedResponse := readLongAddressCommand deserialize: encodedResponse.
	
	"verify"
	self assert: (decodedResponse data size) equals: 2.
	
	firstElement := decodedResponse data at: 1.
	self assert: firstElement equals: 1.
	
	array := (decodedResponse data at: 2).
	self assert: array size equals: 2.
	self assert: (array at: 1) equals: 1.
	self assert: (array at: 2) equals: 2.

! !

!CommandTest methodsFor: 'execution' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:38'!
testSendingNameCommandToThermalSubsystem
	| thermalSubsystem command response |
	"setup"
	thermalSubsystem := SatelliteTest thermalSubsystem.
	command := GetNameCommand newWithReceiver: thermalSubsystem andId: 0.
	
	"exercise"
	response := command execute.
	
	"verify"
	self assert: (response message) equals: 'THERMAL'! !

!CommandTest methodsFor: 'execution' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSendingSetMatrixKeyCommandToThermalSubsystem
	| thermalSubsystem command response voidResponse |
	"setup"
	thermalSubsystem := SatelliteTest thermalSubsystem.
	command := ThermalSetMatrixKeyCommand newWithReceiver: thermalSubsystem andId: 0.
	voidResponse := VoidCommandResponse new.	
	"exercise"
	command addParameterValue: 10.
	response := command execute.
	"verify"
	self assert: response equals: voidResponse.
	self assert: (thermalSubsystem getValueOfAttribute: 'matrix_key') equals: 10! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSerializingCommandWithFiveU32ParametersShouldSerializeOK
	
	| command |
	
	command := MemoryReadChunkedAddressCommand newWithReceiver: SatelliteTest memorySubsystem andId: 45.

	command addParameterValue: 1.
	command addParameterValue: 2.
	command addParameterValue: 3.
	command addParameterValue: 4.
	command addParameterValue: 5.
	
	self assert: command serialize equals: #[45 0 0 0 1 0 0 0 2 0 0 0 3 0 0 0 4 0 0 0 5]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:54'!
testSerializingCommandWithMaxValueSetToOneU24ParameterShouldSerializeOK
	
	| command |
	
	command := PowerStbSetSwitchesStateCommand newWithReceiver: SatelliteTest powerSubsystem andId: 36.

	command addParameterValue: 16777215.
	
	self assert: command serialize equals: #[36 255 255 255]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:40'!
testSerializingCommandWithMaxValueSetToOneU8ParameterShouldSerializeOK
	
	| command |
	
	command := PlatformSetModeCommand newWithReceiver: SatelliteTest platformSubsystem andId: 32.

	command addParameterValue: 255.
	
	self assert: command serialize equals: #[32 255]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSerializingCommandWithMaxValuesSetToTwoU32ParametersShouldSerializeOK
	
	| command |
	
	command := MemoryReadLongAddressCommand newWithReceiver: SatelliteTest memorySubsystem andId: 44.

	command addParameterValue: 4294967295.
	command addParameterValue: 4294967295.
	
	self assert: command serialize equals: #[44 255 255 255 255 255 255 255 255]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:51'!
testSerializingCommandWithOneU24ParameterShouldSerializeOK
	
	| command |
	
	command := PowerStbSetSwitchesStateCommand newWithReceiver: SatelliteTest powerSubsystem andId: 36.

	command addParameterValue: 1.
	
	self assert: command serialize equals: #[36 0 0 1]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSerializingCommandWithOneU32AndOneString8ParametersShouldSerializeOK
	
	| command |
	
	command := MemoryWriteAddressCommand newWithReceiver: SatelliteTest memorySubsystem andId: 44.

	command addParameterValue: 1.
	command addParameterValue: 'Hello'.
	
	self assert: command serialize equals: #[44 0 0 0 1 5 72 101 108 108 111]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSerializingCommandWithOneU32ParameterShouldSerializeOK
	
	| command |
	
	command := ThermalSetMatrixKeyCommand newWithReceiver: SatelliteTest thermalSubsystem andId: 32.

	command addParameterValue: 1.
	
	self assert: command serialize equals: #[32 0 0 0 1]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
testSerializingCommandWithTwoU32ParametersShouldSerializeOK
	
	| command |
	
	command := MemoryReadLongAddressCommand newWithReceiver: SatelliteTest memorySubsystem andId: 44.

	command addParameterValue: 1.
	command addParameterValue: 2.
	
	self assert: command serialize equals: #[44 0 0 0 1 0 0 0 2]! !

!CommandTest methodsFor: 'serialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:01'!
testSerializingCommandWithoutParametersShouldSerializeCommandIdOnly
	
	| command |
	
	command := CommandTest thermalSubsystemTelemetryCommand.
	
	self assert: command serialize equals: #[1]! !

TestCase subclass: #ControllerLogTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ControllerLogTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:13'!
testWhenAskingForASequenceNumberNotLoggedTheResultShouldBeFalse
	| log |
	
	log := ControllerLog new.
	
	self deny: (log containsEntryWithSequenceNumber: 1)
! !

!ControllerLogTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:13'!
testWhenLoggingARequestTheSequenceNumberIsLogged
	| log |
	
	log := ControllerLog new.
	
	log addRequest: 'telemetry' withSequenceNumber: 1.
	
	self assert: (log containsEntryWithSequenceNumber: 1)
! !

!ControllerLogTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:14'!
testWhenLoggingARequestUsingAnExistingSequenceNumberShouldArchiveTheOriginalRequest
	"log using a sequence number and then log again using the same sequence number, to make the log archive the first entry"
	| log newLogEntry logEntryForArchiving archivedLogEntry |
	
	"setup"
	log := ControllerLog new.
	log addRequest: 'messageMovedToTheArchive' withSequenceNumber: 1.
	self assert: (log containsEntryWithSequenceNumber: 1).
	logEntryForArchiving := log entryWithSequenceNumber: 1.
	
	"exercise"
	"this should archive the first entry"
	log addRequest: 'expectedMessage' withSequenceNumber: 1.
	
	"verify"
	newLogEntry := log entryWithSequenceNumber: 1.
	self assert: (newLogEntry request) equals: 'expectedMessage'.
	archivedLogEntry := log archive at: 1.
	self assert: archivedLogEntry equals: logEntryForArchiving.

! !

!ControllerLogTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:14'!
testWhenLoggingAResponseUsingAnExistingSequenceNumberTheLogEntryShouldContainTheRequestAndTheResponse
	| log logEntry |
	
	"setup"
	log := ControllerLog new.
	log addRequest: 'requestMessage' withSequenceNumber: 1.
		
	"exercise"
	log addResponse: 'responseMessage' withSequenceNumber: 1.
	
	"verify"
	logEntry := log entryWithSequenceNumber: 1.
	self assert: (logEntry request) equals: 'requestMessage'.
	self assert: (logEntry response) equals: 'responseMessage'.
! !

TestCase subclass: #SatelliteSubsystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:21'!
testWhenCreatingASubsystemItCanAnswerBasicCommands

	| subsystem |

	"setup"
	subsystem := SatelliteTest memorySubsystem.

	"verify"	
	self assert: (subsystem understands: GetNameCommand).
	self assert: (subsystem understands: ThermalTelemetryCommand).
	self assert: (subsystem understands: ThermalShortTelemetryCommand).
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:24'!
testWhenCreatingASubsystemItDoesntUnderstandUnknwonCommand

	| subsystem notACommand |

	"setup"
	subsystem := SatelliteTest memorySubsystem.
	notACommand := Object new.

	"verify"	
	self deny: (subsystem understands: notACommand).
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:15'!
testWhenCreatingASubsystemTheIdAndNameIsSet

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self assert: subsystem id equals: 1.
	self assert: subsystem name equals: 'MEMORY'.
! !

TestCase subclass: #SatelliteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteTest class methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:53'!
memorySubsystem
	| memorySubsystem |

	memorySubsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.
	memorySubsystem setAttribute: 'free' withValue: 100.
	
	^ memorySubsystem! !

!SatelliteTest class methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:27'!
platformSubsystem
	| platformSubsystem |

	platformSubsystem := PlatformSatelliteSubsystem newWithId: 0 andName: 'PLATFORM'.
	platformSubsystem setAttribute: 'tick_count' withValue: 1000000.
	platformSubsystem setAttribute: 'current_mode' withValue: 1.
	
	^ platformSubsystem! !

!SatelliteTest class methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:52'!
powerSubsystem
	| platformSubsystem |

	platformSubsystem := PowerSatelliteSubsystem newWithId: 2 andName: 'POWER'.
	
	^ platformSubsystem! !

!SatelliteTest class methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:27'!
satelliteWithAllSubsystems
	| subsystems |

	subsystems := Set new.
	subsystems add: self platformSubsystem.
	subsystems add: self memorySubsystem.
	subsystems add: self thermalSubsystem.

	^ Satellite newWithSubsystems: subsystems.
! !

!SatelliteTest class methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 14:53'!
thermalSubsystem
	| thermalSubsystem |

	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem setAttribute: 'battery_temp' withValue: 10.
	thermalSubsystem setAttribute: 'radio_temp' withValue: 20.
	
	^ thermalSubsystem! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 18:21'!
testDecodeThermalSubsystemNameCommand

	| satellite bytes getNameCommand commandEncoder decodedCommand |

	"setup"
	satellite := SatelliteTest satelliteWithAllSubsystems.
	commandEncoder := CommandEncoder new.
	getNameCommand := GetNameCommand newWithReceiver: SatelliteTest thermalSubsystem andId: 0.
	bytes := commandEncoder encodeWith: 'FFF' andSequenceNumber: 1 andCommand: getNameCommand.

	"exercise"	
	decodedCommand := satellite receive: bytes.

	"verify"	
	self assert: decodedCommand equals: getNameCommand.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:27'!
testGivenASatelliteWhenEmittingAShortBeaconItReturnsAShortBeacon

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := SatelliteTest thermalSubsystem.
	thermalSubsystem setAttribute: 'battery_temp' withValue: '10'.
	thermalSubsystem setAttribute: 'radio_temp' withValue: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"short_telemetry":[{"subsystem_id":"3","battery_temp":"10"}]}'.

	"exercise"	
	beacon := satellite emitShortBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
testGivenASatelliteWithThermalSubsystemOnlyWhenEmittingABeaconTheBeaconContainsOnlyThermalTelemetry

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem setAttribute: 'battery_temp' withValue: '10'.
	thermalSubsystem setAttribute: 'radio_temp' withValue: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"telemetry":[{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:24'!
testGivenASatelliteWithTwoSubsystemsWhenEmittingABeaconTheBeaconContainsTwoTelemetryData

	| satellite expectedBeacon beacon |

	"setup"
	satellite := SatelliteTest satelliteWithAllSubsystems.

	expectedBeacon := '{"telemetry":[{"subsystem_id":"1","free":"100"},{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

TestCase subclass: #SequenceNumberTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SequenceNumberTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
test1AsByteArray
	| sequenceNumber |

	sequenceNumber := SequenceNumber new.

	self assert: sequenceNumber asByteArray equals: #[0 0 1].! !

!SequenceNumberTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
testLastSequenceNumberAsByteArray
	| sequenceNumber lastNumber |
	
	lastNumber := LargePositiveInteger readFrom: 'FFFFF0' base: 16.

	sequenceNumber := SequenceNumber startingFrom: lastNumber.

	self assert: sequenceNumber asByteArray equals: #[255 255 240].! !

!SequenceNumberTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
testNewSequenceNumberStartsIn1
	| sequenceNumber |
	
	sequenceNumber := SequenceNumber new.
	
	self assert: sequenceNumber number equals: 1! !

!SequenceNumberTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 15:50'!
testNextOfLastNumberIs1
	| lastNumber sequenceNumber one |
	
	lastNumber := LargePositiveInteger readFrom: 'FFFFEF' base: 16.
	sequenceNumber := SequenceNumber startingFrom: lastNumber.
	one := sequenceNumber next.
	
	self assert: one equals: 1! !

Object subclass: #Command
	instanceVariableNames: 'receiver id parameters responseParameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!Command class methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/15/2013 20:28'!
newWithReceiver: aReceiver andId: anId
	^ self new initializeWithReceiver: aReceiver andId: anId ! !

!Command class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/16/2013 10:00'!
newWithReceiver: aReceiver andId: anId andParameterSizes: aList
	^ self new initializeWithReceiver: aReceiver andId: anId andParameterSizes: aList! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 13:03'!
addParameterValue: aValue
	| parameter |
	
	parameter := parameters at: (parameters findFirst: [ :p | (p value) isNil]).
	
	parameter value: aValue.
! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 19:54'!
deserialize: serializedResponse 

	| responseData readStream |

	responseParameters isEmpty ifTrue: [ ^VoidCommandResponse new. ].
		
	"discard first 3 bytes of sequence number"
	readStream := ReadStream on: (serializedResponse allButFirst: 3).

	responseData := OrderedCollection new.

	responseParameters do: [ :parameter | responseData add: (parameter deserialize: readStream)].
	
	^ DataCommandResponse newWithData: responseData.
! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 15:11'!
execute
	^ self subclassResponsibility.! !

!Command methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/15/2013 17:46'!
id
	^ id! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:44'!
initialize
	self initializeCommandDefinition ! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:44'!
initializeCommandDefinition
	parameters := OrderedCollection new.
	responseParameters := OrderedCollection new.! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 12:48'!
initializeWithReceiver: aReceiver andId: anId

	receiver := aReceiver.
	id := anId.
	self initializeCommandDefinition.! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 18:25'!
receiver
	^ receiver! !

!Command methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:01'!
serialize
	
	| stream |
			
	stream := WriteStream on: ByteArray new.
	
	stream nextPut: (id asByte).

	parameters do: [ :parameter |
		stream nextPutAll: (parameter serialize).
	].
	
	^ stream contents.! !

Command subclass: #GetNameCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!GetNameCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 15:12'!
execute
	^ MessageCommandResponse newWithMessage: (receiver name)! !

!GetNameCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 19:45'!
initializeCommandDefinition
	parameters := OrderedCollection new.
		
	responseParameters := OrderedCollection new.
	responseParameters add: StringCommandParameter new.
! !

Command subclass: #MemoryReadAddressCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemoryReadAddressCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:08'!
initializeCommandDefinition
	parameters := OrderedCollection new.
	parameters add: (UnsignedIntegerCommandParameter new: 32).
	
	responseParameters := OrderedCollection new.
	responseParameters add: (ArrayCommandParameter newWithElementType: UnsignedIntegerCommandParameter andElementSize: 8 andLength: 200).
! !

Command subclass: #MemoryReadChunkedAddressCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemoryReadChunkedAddressCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:14'!
initializeCommandDefinition
	| parameter |
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	responseParameters := OrderedCollection new.! !

Command subclass: #MemoryReadLongAddressCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemoryReadLongAddressCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:14'!
initializeCommandDefinition
	| parameter |
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.

	responseParameters := OrderedCollection new.
	responseParameters add: (UnsignedIntegerCommandParameter new: 32).
	responseParameters add: (ArrayCommandParameter newWithElementType: UnsignedIntegerCommandParameter andElementSize: 8 andLength: 200).
! !

Command subclass: #MemoryWriteAddressCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemoryWriteAddressCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:13'!
initializeCommandDefinition
	| parameter |
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.
	
	parameter := StringCommandParameter new: 8.
	parameters add: parameter.

	responseParameters := OrderedCollection new.! !

Command subclass: #PlatformSetModeCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!PlatformSetModeCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:13'!
initializeCommandDefinition
	| parameter |
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 8.
	parameters add: parameter.

	responseParameters := OrderedCollection new.! !

Command subclass: #PowerStbSetSwitchesStateCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!PowerStbSetSwitchesStateCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:13'!
initializeCommandDefinition
	| parameter |
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 24.
	parameters add: parameter.
	
	responseParameters := OrderedCollection new.! !

Command subclass: #ThermalSetMatrixKeyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalSetMatrixKeyCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 12:57'!
execute
	receiver setMatrixKey: ((parameters at: 1) value).
	
	^ VoidCommandResponse new.
	! !

!ThermalSetMatrixKeyCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:13'!
initializeCommandDefinition
	| parameter |
	
	parameters := OrderedCollection new.
	
	parameter := UnsignedIntegerCommandParameter new: 32.
	parameters add: parameter.


	responseParameters := OrderedCollection new.! !

Command subclass: #ThermalShortTelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalShortTelemetryCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:23'!
initializeCommandDefinition
	parameters := OrderedCollection new.
	
	responseParameters := OrderedCollection new.
	responseParameters add: (UnsignedIntegerCommandParameter new: 8).
! !

Command subclass: #ThermalTelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalTelemetryCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:42'!
initialize
	self initializeCommandDefinition.
! !

!ThermalTelemetryCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 15:15'!
initializeCommandDefinition 
	self initializeParametersDefinition.
	self initializeResponseParametersDefinition.
! !

!ThermalTelemetryCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 15:15'!
initializeParametersDefinition 
	parameters := OrderedCollection new.
! !

!ThermalTelemetryCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 15:16'!
initializeResponseParametersDefinition 
	| parameter |

	responseParameters := OrderedCollection new.

	parameter := SignedIntegerCommandParameter new: 16.
	responseParameters add: parameter.

	parameter := SignedIntegerCommandParameter new: 16.
	responseParameters add: parameter.
! !

Object subclass: #CommandEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!CommandEncoder methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/19/2013 23:15'!
encodeWith: aMAC andSequenceNumber: aSequenceNumber andCommand: aCommand
	| subsystemId encodedCommand encodedBytes stream |
	
	subsystemId := aCommand receiver id.

	encodedCommand := aCommand serialize.
	
	encodedBytes := ByteArray new: (aMAC asByteArray size) + (aSequenceNumber asByteArray size) + (subsystemId asByteArray size) + (encodedCommand size).

	stream := WriteStream on: encodedBytes.
	
	stream nextPutAll: aMAC asByteArray.
	stream nextPutAll: aSequenceNumber asByteArray.
	stream nextPutAll: subsystemId asByteArray.
	stream nextPutAll: encodedCommand.
	
	^ encodedBytes.! !

Object subclass: #CommandParameter
	instanceVariableNames: 'value bitSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

CommandParameter subclass: #ArrayCommandParameter
	instanceVariableNames: 'type arrayLength elementSize elementType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ArrayCommandParameter class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:08'!
newWithElementType: type andElementSize: elementSize andLength: arrayLength
	^ self new initializeWithElementType: type andElementSize: elementSize andArrayLength: arrayLength.
	! !

!ArrayCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:06'!
deserialize: aReadStream 
	| result responseParameter |
	
	result := OrderedCollection new.
	
	[aReadStream atEnd] whileFalse: [ 
		" create an object of type elementType (i.e. UnsignedIntegerCommandParameter) and size in bits elementSize"
		responseParameter := elementType new: elementSize.
		value := responseParameter deserialize: aReadStream.
		result add: value.
	 ].

	^ result.! !

!ArrayCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 17:51'!
elementSize
	^ elementSize! !

!ArrayCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:07'!
initializeWithElementType: theType andElementSize: theElementSize andArrayLength: theArrayLength
	elementType := theType.
	elementSize := theElementSize.
	arrayLength := theArrayLength.! !

!CommandParameter class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:03'!
new: aSize
	^ self new initializeWithBitSize: aSize! !

!CommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:04'!
byteSize
	^ (bitSize / 8)! !

!CommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:04'!
initializeWithBitSize: aSize
	bitSize := aSize! !

!CommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:22'!
value
	^ value! !

!CommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:22'!
value: aValue
	value := aValue! !

CommandParameter subclass: #SignedIntegerCommandParameter
	instanceVariableNames: 'size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SignedIntegerCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 16:31'!
deserialize: aReadstream 
	| serializedValue sign result |
	
	"take first 2 bytes from the input, first byte is sign"

	sign := aReadstream next asInteger.
	
	serializedValue := aReadstream next.
	
	result := serializedValue asInteger.
	
	(sign = 1) ifTrue: [ result := result * -1].
	
	^ result.! !

CommandParameter subclass: #StringCommandParameter
	instanceVariableNames: 'size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!StringCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 19:49'!
deserialize: aReadStream 

	^ aReadStream contents asString.
! !

!StringCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:24'!
serialize 

	| valueBytes valueSize headerSize header result stream |

	valueBytes := value asByteArray.
	valueSize := valueBytes size.

	headerSize := (self byteSize).	
	header := ByteArray new: headerSize.
	header replaceFrom: (headerSize - (valueSize asByteArray size) + 1) to: headerSize with: (valueSize asByteArray).
	
	result := ByteArray new: headerSize + valueSize.
	stream := WriteStream on: result.
	stream nextPutAll: header.
	stream nextPutAll: valueBytes.

	^ result.
! !

CommandParameter subclass: #UnsignedIntegerCommandParameter
	instanceVariableNames: 'size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!UnsignedIntegerCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/22/2013 18:02'!
deserialize: aReadstream 
	| serializedValue |
	
	serializedValue := aReadstream next: (self byteSize).
	
	^ serializedValue asInteger.
! !

!UnsignedIntegerCommandParameter methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 14:24'!
serialize

	| valueBytes start stop result |
	
	result := ByteArray new: (self byteSize).
	
	valueBytes := value asByteArray.
	start := (result size) - (valueBytes size) + 1.
	stop := (result size).
	result replaceFrom: start to: stop with: valueBytes.
	
	^ result.
! !

Object subclass: #CommandResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

CommandResponse subclass: #DataCommandResponse
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!DataCommandResponse class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:23'!
newWithData: data
	^ self new initializeWithData: data.! !

!DataCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:28'!
data
	^ data! !

!DataCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:24'!
initializeWithData: aData
	data := aData! !

CommandResponse subclass: #MessageCommandResponse
	instanceVariableNames: 'message sequenceNumber contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MessageCommandResponse class methodsFor: 'class initialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 20:08'!
newWithMessage: aMessage 
	^ self new initializeWithMessage: aMessage.! !

!MessageCommandResponse class methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/17/2013 18:10'!
newWithSequenceNumber: aSequenceNumber andContents: anOrderedCollection 

	^self new initializeWith: aSequenceNumber andContents: anOrderedCollection ! !

!MessageCommandResponse class methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/16/2013 20:32'!
newWithSequenceNumber: aSequenceNumber andMessage: aMessage 
	^ self new initializeWith: aSequenceNumber and: aMessage! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:01'!
= anObject
	^ message = anObject message
! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/17/2013 18:12'!
at: aSmallInteger

	^ contents at: aSmallInteger ! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:02'!
initialize
	message = ''
! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/16/2013 20:33'!
initializeWith: aSequenceNumber and: aMessage 
	sequenceNumber := aSequenceNumber.
	message := aMessage.! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/17/2013 18:11'!
initializeWith: aSequenceNumber andContents: anOrderedCollection 
		sequenceNumber := aSequenceNumber.
		contents := anOrderedCollection.! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:06'!
initializeWithMessage: aMessage 
	message := aMessage.! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 21:02'!
message
	^ message
! !

!MessageCommandResponse methodsFor: 'as yet unclassified' stamp: 'EzequielDavidovichCaballero 9/16/2013 20:34'!
sequenceNumber

	^ sequenceNumber! !

CommandResponse subclass: #VoidCommandResponse
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:58'!
= anotherResponse
	"TODO reemplazar con una sola instancia global de VoidCommandResponse"
	^ code = anotherResponse code ! !

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:58'!
code
	^ code! !

!VoidCommandResponse methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 20:56'!
initialize
	code = -1
! !

Object subclass: #ControllerLog
	instanceVariableNames: 'entries archive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ControllerLog class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:13'!
newWithArchive: aLogArchive 
	^ self new initializeWithArchive: aLogArchive.! !

!ControllerLog methodsFor: 'recording' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 22:51'!
addRequest: aRequest withSequenceNumber: aSequenceNumber
	| entry |

	(self containsEntryWithSequenceNumber: aSequenceNumber) ifTrue: [
		entry := self entryWithSequenceNumber: aSequenceNumber.
		self archive: entry. ].
	
	entry := ControllerLogEntry newWithRequest: aRequest andSequenceNumber: aSequenceNumber.
	
	entries at: aSequenceNumber put: entry.
! !

!ControllerLog methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 22:21'!
addResponse: aResponse withSequenceNumber: aSequenceNumber
	| logEntry |

	logEntry := entries at: aSequenceNumber.

	logEntry setResponse: aResponse.
	! !

!ControllerLog methodsFor: 'recording' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:12'!
archive
	^ archive! !

!ControllerLog methodsFor: 'recording' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:12'!
archive: anEntry
	archive add: anEntry.! !

!ControllerLog methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 22:51'!
containsEntryWithSequenceNumber: aNumber 
	[entries at: aNumber] on: KeyNotFound do: [ ^ false ].
	^ true.! !

!ControllerLog methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:15'!
entryWithSequenceNumber: aSequenceNumber 
	^ entries at: aSequenceNumber.! !

!ControllerLog methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 23:12'!
initialize
	entries := Dictionary new.
	archive := LinkedList new.! !

Object subclass: #ControllerLogEntry
	instanceVariableNames: 'sequenceNumber request time response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

ControllerLogEntry class
	instanceVariableNames: 'time sequenceNumber request'!

ControllerLogEntry class
	instanceVariableNames: 'time sequenceNumber request'!

!ControllerLogEntry class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:46'!
newWithRequest: aRequest andSequenceNumber: aSequenceNumber
	^ self new initializeWithRequest: aRequest andSequenceNumber: aSequenceNumber.! !

!ControllerLogEntry methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:34'!
initializeWithRequest: aRequest andSequenceNumber: aSequenceNumber
	time := DateAndTime now.
	request := aRequest.
	sequenceNumber := aSequenceNumber.! !

!ControllerLogEntry methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:42'!
request
	^ request
! !

!ControllerLogEntry methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 22:20'!
response
	^ response! !

!ControllerLogEntry methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 22:21'!
setResponse: aResponse
	response := aResponse.! !

!ControllerLogEntry methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/17/2013 20:35'!
time
	^ time! !

Object subclass: #MacGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MacGenerator class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
for: aSerializedCommandCall with: aSequenceNumber

	^ self new initializeFor: aSerializedCommandCall with: aSequenceNumber! !

!MacGenerator class methodsFor: 'key' stamp: 'HernanWilkinson 3/25/2013 14:57'!
key

	self subclassResponsibility ! !

!MacGenerator methodsFor: 'initialization' stamp: 'HernanWilkinson 3/25/2013 14:53'!
initializeFor: aSerializedCommandCall with: aSequenceNumber 

	self subclassResponsibility ! !

!MacGenerator methodsFor: 'evaluating' stamp: 'NahuelGarbezza 5/18/2013 09:40'!
key

	^ self class key! !

!MacGenerator methodsFor: 'evaluating' stamp: 'HernanWilkinson 3/25/2013 14:53'!
value

	self subclassResponsibility ! !

Object subclass: #Satellite
	instanceVariableNames: 'subsystems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!Satellite class methodsFor: 'instance creation' stamp: 'EmilianoMenendez 9/7/2013 16:27'!
newWithSubsystems: theSubsystems 
	^ self new initalizeWithSubsystems: theSubsystems! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:48'!
emitBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem telemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 21:06'!
emitShortBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"short_telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem shortTelemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'initialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 16:11'!
initalizeWithSubsystems: theSubsystems

	subsystems := theSubsystems.


! !

!Satellite methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:59'!
subsystems
	
	^ subsystems
! !

Object subclass: #SatelliteSubsystem
	instanceVariableNames: 'id data telemetryKeys shortTelemetryKeys name commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteSubsystem subclass: #MemorySatelliteSubsystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemorySatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:00'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'free'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'free'.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys.
	
	^ subsystem.! !

SatelliteSubsystem subclass: #PlatformSatelliteSubsystem
	instanceVariableNames: 'matrixKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!PlatformSatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:30'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys platformSetModeCommand platformSetModeCommandEntry |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'tick_count'.
	telemetryKeys add: 'current_mode'.
	
	shortTelemetryKeys := Set new.
	
	platformSetModeCommand := PlatformSetModeCommand new.
	platformSetModeCommandEntry := Association key: 'setMode' value: platformSetModeCommand.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys andCommands: platformSetModeCommandEntry.
	
	^ subsystem.! !

!PlatformSatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:31'!
setMode: theValue
	data at: 'current_mode' put: theValue! !

SatelliteSubsystem subclass: #PowerSatelliteSubsystem
	instanceVariableNames: 'matrixKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!PowerSatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:49'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys powerStbSetSwitchesStateCommand powerStbSetSwitchesStateCommandEntry |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'low_voltage_counter'.
	telemetryKeys add: 'battery_v'.
	telemetryKeys add: 'array_X_v'.
	telemetryKeys add: 'array_Xp_mA'.
	telemetryKeys add: 'array_Xm_mA'.
	telemetryKeys add: 'array_Y_v'.
	telemetryKeys add: 'array_Yp_mA'.
	telemetryKeys add: 'array_Y_mA'.
	telemetryKeys add: 'array_Z_v'.
	telemetryKeys add: 'array_Zp_mA'.
	telemetryKeys add: 'array_Zm_mA'.
	telemetryKeys add: '_3v3_mA'.
	telemetryKeys add: '_5v_mA'.

	shortTelemetryKeys := Set new.
	
	powerStbSetSwitchesStateCommand := PowerStbSetSwitchesStateCommand new.
	powerStbSetSwitchesStateCommandEntry := Association key: 'stbSetSwitchesState' value: powerStbSetSwitchesStateCommand.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys andCommands: powerStbSetSwitchesStateCommandEntry.
	
	^ subsystem.! !

!PowerSatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:42'!
setMode: theValue
	data at: 'current_mode' put: theValue! !

!SatelliteSubsystem class methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:21'!
basicCommands
	| commands |
	
	commands := Set new.

	commands add: GetNameCommand.
	commands add: ThermalTelemetryCommand.
	commands add: ThermalShortTelemetryCommand.

	^ commands! !

!SatelliteSubsystem class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:58'!
newWithId: theId andName: theName
	^ self new initializeWithId: theId andName: theName andTelemetryKeys: Set new andShortTelemetryKeys: Set new! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:35'!
= anObject
	^ id = anObject id! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:27'!
commands
	^ commands! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:03'!
getValueOfAttribute: aKey
	^ (data at: aKey)! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:10'!
id
	^ id! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:26'!
initializeWithId: theId andName: theName andTelemetryKeys: theTelemetryKeys andShortTelemetryKeys: theShortTelemetryKeys
	id := theId.
	name := theName.
	commands := SatelliteSubsystem basicCommands.
	telemetryKeys := theTelemetryKeys.
	shortTelemetryKeys := theShortTelemetryKeys.
	data := Dictionary new.! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/10/2013 19:37'!
initializeWithId: theId andName: theName andTelemetryKeys: theTelemetryKeys andShortTelemetryKeys: theShortTelemetryKeys andCommands: theCommands
	id := theId.
	name := theName.
	commands := SatelliteSubsystem basicCommands.
	commands add: theCommands.
	telemetryKeys := theTelemetryKeys.
	shortTelemetryKeys := theShortTelemetryKeys.
	data := Dictionary new.! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:11'!
name
	^ name! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 12:46'!
setAttribute: aKey withValue: aValue 
	data at: aKey put: aValue! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendez 9/9/2013 10:26'!
shortTelemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | (shortTelemetryKeys includes: (entry key)) ifTrue: [temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.]].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'command' stamp: 'EmilianoMenendez 9/8/2013 20:40'!
telemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:22'!
understands: aCommandClass
	^ (commands includes: aCommandClass).! !

SatelliteSubsystem subclass: #ThermalSatelliteSubsystem
	instanceVariableNames: 'matrixKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalSatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/21/2013 17:20'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys matrixKeyCommand matrixKeyCommandEntry |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'battery_temp'.
	telemetryKeys add: 'radio_temp'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'battery_temp'.
	
	matrixKeyCommand := ThermalSetMatrixKeyCommand new.
	matrixKeyCommandEntry := Association key: 'setMatrixKey' value: matrixKeyCommand.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys andCommands: matrixKeyCommandEntry.
	
	^ subsystem.! !

!ThermalSatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/14/2013 16:07'!
setMatrixKey: theValue
	data at: 'matrix_key' put: theValue! !

Object subclass: #SequenceNumber
	instanceVariableNames: 'number lastNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SequenceNumber class
	instanceVariableNames: 'number'!

SequenceNumber class
	instanceVariableNames: 'number'!

!SequenceNumber class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/20/2013 17:36'!
startingFrom: aNumber
	^ self new initializeWithStart: aNumber! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 19:08'!
asByteArray
	| resultSize resultBytes sequenceNumber start |

	resultSize := 3.

	resultBytes := ByteArray new: resultSize.
	
	sequenceNumber := number asByteArray.

	start := (resultSize - (sequenceNumber size) + 1).

	^ resultBytes replaceFrom: start to: resultSize with: sequenceNumber. 
! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 21:14'!
initialize
	number := 1.
	lastNumber := LargePositiveInteger readFrom: 'FFFFEF' base: 16.
! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 14:58'!
initializeWithStart: aNumber
	self initialize.
	number := aNumber.! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 14:53'!
next
	number := number + 1.
	number > lastNumber ifTrue: [ self reset. ].
	^ number.
! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 15:00'!
number
	^ number! !

!SequenceNumber methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/15/2013 14:52'!
reset
	number := 1.! !
